var controllerFactory = require("./controllerfactory");

/*
  Module is responsible for looking up content matching the current request + fetching the root of the site, as well as the standard top-navigation
*/

module.exports = {
  handle: function(umbracoClient) {
    return async function(req, res, next) {
      var self = this;

      if (!umbracoClient) {
        throw new Error("Umbraco Client not present");
      }

      //get the path
      var path = req.path;

      //get the current method, eg: post,get,put,delete
      var method = req.method.toLowerCase();

      //detect if a umbDebug querystring is there - if so - return as a raw json object for debugging
      var serveAsJson = req.query.umbDebug;
      req.contentContext = {
        umbraco: umbracoClient,
        serveAsJson: serveAsJson,
        model: { query: {} }
      };

      //get the root of the website
      var root = await umbracoClient.content.site(); //returns a representation and a resource

      //if the current page is the root - just make current equal root - otherwise try fetching the page matching the url
      var current =
        path === "/" ? root : await umbracoClient.content.byUrl(path); // returns a representation and a resource

      //if a page was found matching the url - try finding a matching controller to handle this type of content
      //eg - if the page is of type "home" - look for a "homecontroller"
      //if the request method is GET - try to run homecontroller.get and so forth
      if (current !== null && current.data != null) {
        //prepare the current page for attached queries
        var model = current.data.body;
        model.query = {};

        //attach the root node on the query object for easy fetching
        model.query.root = root.data.body;

        //attact a topnavigation array to the query array for easy fetching
        model.query.topNavigation = Object.values(
          (await umbracoClient.content.children(root.resource)).data
            .embedded
        );

        //set the model on the contentContext which is carried on to the controller
        req.contentContext.model = model;
        req.contentContext.view = model.contentTypeAlias.toLowerCase();

        //use the controllerFactory to find a controller - if no controller is found - it will return the umbracobasecontroller
        var contentController = controllerFactory.findContentController(
          model.contentTypeAlias,
          method
        );

        contentController(req, res, next);
      } else {
        //if no content is found - try looking for a custom controller for this path and method
        //if the path is /debug - it will look for debugcontroller
        //if path is /debug/action - it will look for debug.actioncontroller.js
        var customController = controllerFactory.findCustomController(
          path,
          method
        );

        //if a custom controller is found - use it to do whatevs
        if (customController) {
          customController(req, res, next);
        } else {
          res.render("notfound", { url: path });
        }
      }
    };
  }
};
